/* Copyright (c) 2022 Daniele Lombardi / Daniels118
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package it.ld.sh3d.devtoolkit;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import com.eteks.sweethome3d.plugin.PluginAction;

/**This class is a PluginAction that proxies method invocations to another PluginAction.
 * @author Daniels118
 */
class DynamicAction extends PluginAction {
	private PluginAction action;
	
	/**
	 * @param action the PluginAction to which method invocations are proxied
	 */
	public DynamicAction(PluginAction action) {
		setAction(action);
	}
	
	private final PropertyChangeListener propertyAligner = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			Property property = Property.valueOf(evt.getPropertyName());
			putPropertyValue(property, evt.getNewValue());
		}
	};
	
	/**Changes the PluginAction to which method invocations are proxied.
	 * @param newAction
	 */
	void setAction(PluginAction newAction) {
		if (action != null) {
			action.removePropertyChangeListener(propertyAligner);
		}
		this.action = newAction;
		if (action != null) {
			action.addPropertyChangeListener(propertyAligner);
			for (Property property : Property.values()) {
				this.putPropertyValue(property, action.getPropertyValue(property));
			}
		}
	}

	@Override
	public void execute() {
		action.execute();
	}
	
	@Override
	public void putPropertyValue(Property property, Object value) {
		super.putPropertyValue(property, value);
		action.putPropertyValue(property, value);
	}
	
	@Override
	public boolean isEnabled() {
		return action.isEnabled();
	}
	
	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		action.setEnabled(enabled);
	}
}
